@@ -3,7 +3,7 @@
   FILE: icaltime.c
   CREATOR: eric 02 June 2000
   
-  $Id: icaltime.c,v 1.39 2005/09/05 05:08:10 pchen Exp $
+  $Id: icaltime.c,v 1.40 2005/11/28 03:17:39 tml Exp $
   $Locker:  $
     
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
@@ -45,7 +45,7 @@
 #include "icalvalue.h"
 
 #ifdef WIN32
-#include <Windows.h>
+#include <windows.h>
 
 #define snprintf      _snprintf
 #define strcasecmp    stricmp
@@ -414,8 +414,14 @@ time_t icaltime_as_timet_with_zone(const struct icaltimetype tt,
 
     /* Set TZ to UTC and use mktime to convert to a time_t. */
     old_tz = set_tz ("UTC");
+#ifdef WIN32
+    tzset ();
+#endif
     t = mktime (&stm);
     unset_tz (old_tz);
+#ifdef WIN32
+    tzset ();
+#endif
 
     return t;
 }
