@@ -5,16 +5,16 @@
  *  Copyright 2005 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
+ *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
+ *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
@@ -25,12 +25,11 @@
 #include <config.h>
 #endif
 
-#include <camel/camel-i18n.h>
-#include <camel/camel-folder.h>
-#include <camel/camel-session.h>
-
-#include "camel-offline-store.h"
+#include "camel-folder.h"
+#include "camel-i18n.h"
 #include "camel-offline-folder.h"
+#include "camel-offline-store.h"
+#include "camel-session.h"
 
 
 static void camel_offline_store_class_init (CamelOfflineStoreClass *klass);
@@ -102,44 +101,57 @@ offline_store_construct (CamelService *service, CamelSession *session,
 }
 
 
+/**
+ * camel_offline_store_set_network_state:
+ * @store: a #CamelOfflineStore object
+ * @state: the network state
+ * @ex: a #CamelException
+ *
+ * Set the network state to either #CAMEL_OFFLINE_STORE_NETWORK_AVAIL
+ * or #CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL.
+ **/
 void
 camel_offline_store_set_network_state (CamelOfflineStore *store, int state, CamelException *ex)
 {
 	CamelException lex;
-	
+	CamelService *service = CAMEL_SERVICE (store);
+	gboolean network_state = camel_session_get_network_state (service->session);
+
 	if (store->state == state)
 		return;
 	
 	camel_exception_init (&lex);
 	if (store->state == CAMEL_OFFLINE_STORE_NETWORK_AVAIL) {
 		/* network available -> network unavailable */
-		if (((CamelStore *) store)->folders) {
-			GPtrArray *folders;
-			CamelFolder *folder;
-			int i, sync;
-			
-			sync = camel_url_get_param (((CamelService *) store)->url, "sync_offline") != NULL;
-			
-			folders = camel_object_bag_list (((CamelStore *) store)->folders);
-			for (i = 0; i < folders->len; i++) {
-				folder = folders->pdata[i];
+		if (network_state) {
+			if (((CamelStore *) store)->folders) {
+				GPtrArray *folders;
+				CamelFolder *folder;
+				int i, sync;
 				
-				if (CAMEL_CHECK_TYPE (folder, CAMEL_OFFLINE_FOLDER_TYPE)
-				    && (sync || ((CamelOfflineFolder *) folder)->sync_offline)) {
-					camel_offline_folder_downsync ((CamelOfflineFolder *) folder, NULL, &lex);
-					camel_exception_clear (&lex);
+				sync = camel_url_get_param (((CamelService *) store)->url, "sync_offline") != NULL;
+				
+				folders = camel_object_bag_list (((CamelStore *) store)->folders);
+				for (i = 0; i < folders->len; i++) {
+					folder = folders->pdata[i];
+					
+					if (CAMEL_CHECK_TYPE (folder, CAMEL_OFFLINE_FOLDER_TYPE)
+					    && (sync || ((CamelOfflineFolder *) folder)->sync_offline)) {
+						camel_offline_folder_downsync ((CamelOfflineFolder *) folder, NULL, &lex);
+						camel_exception_clear (&lex);
+					}
+					
+					camel_object_unref (folder);
 				}
 				
-				camel_object_unref (folder);
+				g_ptr_array_free (folders, TRUE);
 			}
 			
-			g_ptr_array_free (folders, TRUE);
+			camel_store_sync (CAMEL_STORE (store), FALSE, &lex);
+			camel_exception_clear (&lex);
 		}
-		
-		camel_store_sync (CAMEL_STORE (store), FALSE, &lex);
-		camel_exception_clear (&lex);
-		
-		if (!camel_service_disconnect (CAMEL_SERVICE (store), TRUE, ex))
+
+		if (!camel_service_disconnect (CAMEL_SERVICE (store), network_state, ex))
 			return;
 	} else {
 		/* network unavailable -> network available */
