@@ -123,9 +123,6 @@ groupwise_store_construct (CamelService *service, CamelSession *session,
 	priv->server_name = g_strdup (url->host);
 	priv->user = g_strdup (url->user);
 
-	/*remove the sync option*/
-/*	if (camel_url_get_param ((service->url, "sync_offline") != NULL)
-			camel_url_set_param (service->url, "sync_offline", NULL);*/
 	/*base url*/
 	priv->base_url = camel_url_to_string (service->url, (CAMEL_URL_HIDE_PASSWORD |
 						       CAMEL_URL_HIDE_PARAMS   |
@@ -262,6 +259,7 @@ check_for_connection (CamelService *service, CamelException *ex)
 	return TRUE;
 
 }
+
 static gboolean
 groupwise_connect (CamelService *service, CamelException *ex)
 {
@@ -550,7 +548,7 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 		g_print ("\n\n** %s **: No summary as yet : using get cursor request\n\n", folder->name);
 
 		status = e_gw_connection_create_cursor (priv->cnc, container_id, 
-				"peek id recipient attachments distribution subject status options priority startDate delivered",
+				"peek id recipient attachments distribution subject status options priority startDate created delivered size",
 				NULL,
 				&cursor);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
