@@ -21,24 +21,28 @@
 #include <config.h>
 
 #include <sys/types.h>
-#include <dirent.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <glib.h>
-#include <libsoup/soup-misc.h>
-#include "camel-groupwise-utils.h"
 
-#include <camel/camel-service.h>
-#include <camel/camel-multipart.h>
-#include <camel/camel-stream-mem.h>
-#include <camel/camel-address.h>
-#include <camel/camel-mime-message.h>
+#include <glib.h>
+#include <glib/gstdio.h>
+
+#include <libsoup/soup-misc.h>
+
+#include "camel/camel-address.h"
+#include "camel/camel-mime-message.h"
+#include "camel/camel-multipart.h"
+#include "camel/camel-service.h"
+#include "camel/camel-stream-mem.h"
+
+#include "camel-groupwise-utils.h"
 
 #define SUBFOLDER_DIR_NAME     "subfolders"
 #define SUBFOLDER_DIR_NAME_LEN 10
 #define RFC_822	"message/rfc822"
 
+static void do_multipart (EGwConnection *cnc, EGwItem *item, CamelMultipart *mp, GSList **attach_list);
 /**
  * e_path_to_physical:
  * @prefix: a prefix to prepend to the path, or %NULL
@@ -134,7 +138,7 @@ static gboolean
 find_folders_recursive (const char *physical_path, const char *path,
 			EPathFindFoldersCallback callback, gpointer data)
 {
-	DIR *dir;
+	GDir *dir;
 	char *subfolder_directory_path;
 	gboolean ok;
 
@@ -152,7 +156,7 @@ find_folders_recursive (const char *physical_path, const char *path,
 	}
 
 	/* Now scan the subfolders and load them. */
-	dir = opendir (subfolder_directory_path);
+	dir = g_dir_open (subfolder_directory_path, 0, NULL);
 	if (dir == NULL) {
 		g_free (subfolder_directory_path);
 		return TRUE;
@@ -161,27 +165,23 @@ find_folders_recursive (const char *physical_path, const char *path,
 	ok = TRUE;
 	while (ok) {
 		struct stat file_stat;
-		struct dirent *dirent;
+		const char *dirent;
 		char *file_path;
 		char *new_path;
 
-		dirent = readdir (dir);
+		dirent = g_dir_read_name (dir);
 		if (dirent == NULL)
 			break;
 
-		if (strcmp (dirent->d_name, ".") == 0 || strcmp (dirent->d_name, "..") == 0)
-			continue;
+		file_path = g_strdup_printf ("%s/%s", subfolder_directory_path, dirent);
 
-		file_path = g_strdup_printf ("%s/%s", subfolder_directory_path,
-					     dirent->d_name);
-
-		if (stat (file_path, &file_stat) < 0 ||
+		if (g_stat (file_path, &file_stat) < 0 ||
 		    ! S_ISDIR (file_stat.st_mode)) {
 			g_free (file_path);
 			continue;
 		}
 
-		new_path = g_strdup_printf ("%s/%s", path, dirent->d_name);
+		new_path = g_strdup_printf ("%s/%s", path, dirent);
 
 		ok = find_folders_recursive (file_path, new_path, callback, data);
 
@@ -189,7 +189,7 @@ find_folders_recursive (const char *physical_path, const char *path,
 		g_free (new_path);
 	}
 
-	closedir (dir);
+	g_dir_close (dir);
 	g_free (subfolder_directory_path);
 
 	return ok;
@@ -235,7 +235,7 @@ e_path_rmdir (const char *prefix, const char *vpath)
 
 	/* Remove the directory itself */
 	physical_path = e_path_to_physical (prefix, vpath);
-	if (rmdir (physical_path) == -1) {
+	if (g_rmdir (physical_path) == -1) {
 		g_free (physical_path);
 		return -1;
 	}
@@ -256,7 +256,7 @@ e_path_rmdir (const char *prefix, const char *vpath)
 		return 0;
 	}
 
-	rmdir (physical_path);
+	g_rmdir (physical_path);
 	g_free (physical_path);
 	return 0;
 }
@@ -285,7 +285,7 @@ add_recipients(GSList *recipient_list, CamelAddress *recipients, int recipient_t
 }
 
 static void 
-send_as_attachment (EGwConnection *cnc, EGwItem *item, CamelStreamMem *content, char *buffer, CamelContentType *type, CamelDataWrapper *dw, const char *filename, GSList **attach_list)
+send_as_attachment (EGwConnection *cnc, EGwItem *item, CamelStreamMem *content, char *buffer, CamelContentType *type, CamelDataWrapper *dw, const char *filename, const char *cid, GSList **attach_list)
 {
 	EGwItemLinkInfo *info = NULL;
 	EGwConnectionStatus status;
@@ -298,6 +298,17 @@ send_as_attachment (EGwConnection *cnc, EGwItem *item, CamelStreamMem *content,
 	attachment = g_new0 (EGwItemAttachment, 1);
 	attachment->contentType = g_strdup_printf ("%s/%s", type->type, type->subtype);
 
+	if (cid) {
+		gchar **t;
+		//strip_lt_gt ((char **)&cid, 2, 3);
+		t = g_strsplit_set (cid,"<>",-1);
+		if (!t[1])
+			attachment->contentid = g_strdup (cid);
+		else 
+			attachment->contentid = g_strdup (t[1]);
+		g_strfreev (t);
+	}
+
 	if (filename) {
 		if (!strcmp (attachment->contentType, "application/pgp-signature")) {
 			char *temp_str;
@@ -319,13 +330,16 @@ send_as_attachment (EGwConnection *cnc, EGwItem *item, CamelStreamMem *content,
 	} else {
 		char *temp_str;
 		int temp_len;
-		temp_str = soup_base64_encode (buffer, len);
-		temp_len = strlen (temp_str);
-		attachment->data = g_strdup (temp_str);
-		attachment->size = temp_len;
-		g_free (temp_str);
-		temp_str = NULL;
-		temp_len = 0;
+		if (!strcmp (attachment->contentType, "multipart/digest")) {
+		} else {
+			temp_str = soup_base64_encode (buffer, len);
+			temp_len = strlen (temp_str);
+			attachment->data = g_strdup (temp_str);
+			attachment->size = temp_len;
+			g_free (temp_str);
+			temp_str = NULL;
+			temp_len = 0;
+		}
 	}
 	
 	if (!strcmp (attachment->contentType, "text/html") || !(strcmp (attachment->contentType, "multipart/alternative"))) {
@@ -345,7 +359,7 @@ send_as_attachment (EGwConnection *cnc, EGwItem *item, CamelStreamMem *content,
 		id = (char *)g_malloc0 (len-1);
 		id = memcpy(id, temp_id+2, len-3);
 
-		status = e_gw_connection_forward_item (cnc, id, "message", TRUE, &temp_item);
+		status = e_gw_connection_forward_item (cnc, id, NULL, TRUE, &temp_item);
 		if (status != E_GW_CONNECTION_STATUS_OK) 
 			g_warning ("Could not send a forwardRequest...continuing without!!\n");
 		else {
@@ -363,6 +377,7 @@ send_as_attachment (EGwConnection *cnc, EGwItem *item, CamelStreamMem *content,
 			info = e_gw_item_get_link_info (temp_item);
 			e_gw_item_set_link_info (item, info);
 		}
+		//attachment->id = g_strdup (id);
 		g_free (id);
 	}
 
@@ -379,7 +394,6 @@ camel_groupwise_util_item_from_message (EGwConnection *cnc, CamelMimeMessage *me
 	char *send_options = NULL;
 	CamelMultipart *mp;
 	GSList *recipient_list = NULL, *attach_list = NULL; 
-	int i;
 
 	/*Egroupwise item*/
 	item = e_gw_item_new_empty ();
@@ -408,69 +422,7 @@ camel_groupwise_util_item_from_message (EGwConnection *cnc, CamelMimeMessage *me
 
 	if (CAMEL_IS_MULTIPART (mp)) {
 		/*contains multiple parts*/
-		guint part_count;
-		
-		part_count = camel_multipart_get_number (mp);
-		for ( i=0 ; i<part_count ; i++) {
-			CamelContentType *type;
-			CamelMimePart *part;
-			CamelStreamMem *content = (CamelStreamMem *)camel_stream_mem_new ();
-			CamelDataWrapper *dw = camel_data_wrapper_new ();
-			const char *disposition, *filename;
-			char *buffer = NULL;
-			char *mime_type = NULL;
-			gboolean is_alternative = FALSE;
-			/*
-			 * XXX:
-			 * Assuming the first part always is the actual message
-			 * and an attachment otherwise.....
-			 */
-			part = camel_multipart_get_part (mp, i);
-			type = camel_mime_part_get_content_type(part);
-			dw = camel_medium_get_content_object (CAMEL_MEDIUM (part));
-			if (type->subtype && !strcmp (type->subtype, "alternative")) {
-				CamelMimePart *temp_part;
-				CamelStreamMem *temp_content = (CamelStreamMem *)camel_stream_mem_new ();
-				temp_part = camel_multipart_get_part ((CamelMultipart *)dw, 1);
-				CamelDataWrapper *temp_dw = camel_data_wrapper_new ();
-				if (temp_part) {
-					is_alternative = TRUE;
-					temp_dw = camel_medium_get_content_object (CAMEL_MEDIUM (temp_part));
-					camel_data_wrapper_write_to_stream(temp_dw, (CamelStream *)temp_content);
-					buffer = g_malloc0 (temp_content->buffer->len+1);
-					buffer = memcpy (buffer, temp_content->buffer->data, temp_content->buffer->len);
-					filename = camel_mime_part_get_filename (temp_part);
-					disposition = camel_mime_part_get_disposition (temp_part);
-					mime_type = camel_data_wrapper_get_mime_type (temp_dw);
-					send_as_attachment (cnc, item, temp_content, buffer, type, temp_dw, filename, &attach_list);
-					g_free (buffer);
-					g_free (mime_type);
-				}
-				camel_object_unref (temp_content);
-				camel_object_unref (temp_dw);
-				camel_object_unref (dw);
-				continue;
-			} 
-			
-			camel_data_wrapper_write_to_stream(dw, (CamelStream *)content);
-			buffer = g_malloc0 (content->buffer->len+1);
-			buffer = memcpy (buffer, content->buffer->data, content->buffer->len);
-			filename = camel_mime_part_get_filename (part);
-			disposition = camel_mime_part_get_disposition (part);
-			mime_type = camel_data_wrapper_get_mime_type (dw);
-
-			if (i == 0 && !strcmp (mime_type, "text/plain") ) {
-				e_gw_item_set_content_type (item, mime_type);
-				e_gw_item_set_message (item, buffer);
-			} else 
-				send_as_attachment (cnc, item, content, buffer, type, dw, filename, &attach_list);
-
-			g_free (buffer);
-			g_free (mime_type);
-			camel_object_unref (content);
-			camel_object_unref (dw);
-		} /*end of for*/
-		
+		do_multipart (cnc, item, mp, &attach_list);
 	} else {
 		/*only message*/
 		CamelStreamMem *content = (CamelStreamMem *)camel_stream_mem_new ();
@@ -489,8 +441,9 @@ camel_groupwise_util_item_from_message (EGwConnection *cnc, CamelMimeMessage *me
 		if (!strcmp(content_type, "text/plain")) {
 			e_gw_item_set_content_type (item, content_type);				
 			e_gw_item_set_message (item, buffer);
-		} else
-			send_as_attachment (cnc, item, content, buffer, type, dw, NULL, &attach_list);	
+		} else {
+			send_as_attachment (cnc, item, content, buffer, type, dw, NULL, NULL, &attach_list);	
+		}
 
 		g_free (buffer);
 		g_free (content_type);
@@ -602,3 +555,95 @@ gw_concat ( const char *prefix, const char *suffix)
 		return g_strdup_printf ("%s%c%s", prefix, '/', suffix);
 }
 
+void
+strip_lt_gt (char **string, int s_offset, int e_offset)
+{
+	char *temp = NULL;
+	int len;
+	
+	temp = g_strdup (*string);
+	len = strlen (*string);
+
+	*string = (char *)g_malloc0 (len-1);
+	*string = memcpy(*string, temp+s_offset, len-e_offset);
+	g_free (temp);
+}
+
+static void
+do_multipart (EGwConnection *cnc, EGwItem *item, CamelMultipart *mp, GSList **attach_list)
+{
+	/*contains multiple parts*/
+	guint part_count;
+	int i;
+
+	part_count = camel_multipart_get_number (mp);
+	for ( i=0 ; i<part_count ; i++) {
+		CamelContentType *type;
+		CamelMimePart *part;
+		CamelStreamMem *content = (CamelStreamMem *)camel_stream_mem_new ();
+		CamelDataWrapper *dw = camel_data_wrapper_new ();
+		const char *disposition, *filename;
+		char *buffer = NULL;
+		char *mime_type = NULL;
+		const char *content_id = NULL;
+		gboolean is_alternative = FALSE;
+		/*
+		 * XXX:
+		 * Assuming the first part always is the actual message
+		 * and an attachment otherwise.....
+		 */
+		part = camel_multipart_get_part (mp, i);
+		type = camel_mime_part_get_content_type(part);
+		if (CAMEL_IS_MULTIPART ((CamelMultipart *)camel_medium_get_content_object (part))) {
+			do_multipart (cnc, item, (CamelMultipart *)camel_medium_get_content_object (part), attach_list);
+			continue;
+		} else 
+		dw = camel_medium_get_content_object (CAMEL_MEDIUM (part));
+		if (type->subtype && !strcmp (type->subtype, "alternative")) {
+			CamelMimePart *temp_part;
+			const char *cid = NULL;
+			CamelStreamMem *temp_content = (CamelStreamMem *)camel_stream_mem_new ();
+			temp_part = camel_multipart_get_part ((CamelMultipart *)dw, 1);
+			CamelDataWrapper *temp_dw = camel_data_wrapper_new ();
+			if (temp_part) {
+				is_alternative = TRUE;
+				temp_dw = camel_medium_get_content_object (CAMEL_MEDIUM (temp_part));
+				camel_data_wrapper_write_to_stream(temp_dw, (CamelStream *)temp_content);
+				buffer = g_malloc0 (temp_content->buffer->len+1);
+				buffer = memcpy (buffer, temp_content->buffer->data, temp_content->buffer->len);
+				filename = camel_mime_part_get_filename (temp_part);
+				disposition = camel_mime_part_get_disposition (temp_part);
+				mime_type = camel_data_wrapper_get_mime_type (temp_dw);
+				cid = camel_mime_part_get_content_id (temp_part);
+				send_as_attachment (cnc, item, temp_content, buffer, type, temp_dw, filename, cid, attach_list);
+				g_free (buffer);
+				g_free (mime_type);
+			}
+			camel_object_unref (temp_content);
+			camel_object_unref (temp_dw);
+			camel_object_unref (dw);
+			continue;
+		} 
+		
+
+		camel_data_wrapper_write_to_stream(dw, (CamelStream *)content);
+		buffer = g_malloc0 (content->buffer->len+1);
+		buffer = memcpy (buffer, content->buffer->data, content->buffer->len);
+		filename = camel_mime_part_get_filename (part);
+		disposition = camel_mime_part_get_disposition (part);
+		mime_type = camel_data_wrapper_get_mime_type (dw);
+		content_id = camel_mime_part_get_content_id (part);
+
+		if (i == 0 && !strcmp (mime_type, "text/plain") ) {
+			e_gw_item_set_content_type (item, mime_type);
+			e_gw_item_set_message (item, buffer);
+		} else {
+			send_as_attachment (cnc, item, content, buffer, type, dw, filename, content_id, attach_list);
+		}
+
+		g_free (buffer);
+		g_free (mime_type);
+		camel_object_unref (content);
+		camel_object_unref (dw);
+	} /*end of for*/
+}
