@@ -36,6 +36,7 @@
 
 /*#include "libedataserver/e-path.h"*/
 #include "libedataserver/e-time-utils.h"
+#include "libedataserver/e-data-server-util.h"
 
 #include "camel-imap-folder.h"
 #include "camel-imap-command.h"
@@ -227,7 +228,7 @@ camel_imap_folder_new (CamelStore *parent, const char *folder_name,
 	const char *short_name;
 	char *summary_file, *state_file;
 
-	if (camel_mkdir (folder_dir, S_IRWXU) != 0) {
+	if (e_util_mkdir_hier (folder_dir, S_IRWXU) != 0) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Could not create directory %s: %s"),
 				      folder_dir, g_strerror (errno));
